import {
    type GetPlayQueueResponse, type NowPlayingEntry,
    type NowPlayingResponse,
    OpenSubsonic,
    type Parameter,
    type Song
} from "$lib/opensubsonic";
import {playbackState} from "$lib/states/playback-state.svelte";

interface QueueState {
    queue: Array<Song>,
    currentIndex: number,
    nextSong: () => Song,
    setSong: (index: number) => void,
    getSong: () => Song,
    firstSong: () => Song,
    findSong: (song: Song) => {found: boolean, index: number},
    addSong: (song: Song) => void,
    replaceQueue: (newQueue: Array<Song>) => void,
    getPlayQueue: (addNowPlaying: boolean) => Promise<void>,
    saveQueue: () => Promise<void>,
}

export const queueState: QueueState = $state({
    queue: new Array<Song>(),
    currentIndex: 0,
    nextSong(): Song {
        this.currentIndex += 1;
        return this.queue[this.currentIndex];
    },
    setSong(index: number): void {
        this.currentIndex = index;
    },
    getSong(): Song {
        return this.queue[this.currentIndex];
    },
    firstSong(): Song {
        this.currentIndex = 0;
        return this.queue[this.currentIndex];
    },
    findSong(searchSong: Song): {found: boolean, index: number} {
        const data = {
            found: false,
            index: 0,
        }
        for (const [index, song] of this.queue.entries()) {
            if (song.id === searchSong.id) {
                data.found = true;
                data.index = index;
                break;
            }
        }
        return data;
    },
    addSong(newSong: Song): void {
        this.queue.push(newSong);
        this.currentIndex = this.queue.length - 1;
    },
    replaceQueue(newQueue: Array<Song>): void {
        console.log(newQueue)
        this.queue = [...newQueue];
        this.currentIndex = 0;
    },
    async getPlayQueue(addNowPlaying: boolean = false): Promise<void> {
        const queueData: GetPlayQueueResponse = await OpenSubsonic.get("getPlayQueue")
        if (queueData && queueData.playQueue.entry) {
            this.queue = queueData.playQueue.entry;
        }
        const nowPlayingData: NowPlayingResponse = await OpenSubsonic.get("getNowPlaying");

        if (nowPlayingData && nowPlayingData.nowPlaying.entry) {
            const userEntries: Array<NowPlayingEntry> = [];
            nowPlayingData.nowPlaying.entry.forEach((entry) => {
                if (entry.username === OpenSubsonic.username) {
                    userEntries.push(entry);
                }
            });
            console.log(userEntries);
            let localClientEntry = undefined;
            userEntries.forEach((entry) => {
                if (entry.playerName === OpenSubsonic.clientName) {
                    localClientEntry = entry;
                }
            })

            if (typeof localClientEntry !== "undefined") {
                if (this.findSong(localClientEntry).found) {
                    this.setSong(this.findSong(localClientEntry).index);
                    playbackState.newSong(this.getSong());
                }
                else if (addNowPlaying) {
                    this.addSong(localClientEntry);
                    playbackState.newSong(this.getSong());
                }
                else if (this.queue.length != 0) {
                    playbackState.newSong(this.firstSong());
                }
            }
            else {
                for (const entry of userEntries) {
                    if (this.findSong(entry).found) {
                        this.setSong(this.findSong(entry).index);
                        playbackState.newSong(this.getSong());
                    }
                    else if (addNowPlaying) {
                        this.addSong(entry);
                        playbackState.newSong(this.getSong());
                    }
                    else if (this.queue.length != 0) {
                        playbackState.newSong(this.firstSong());
                    }
                }
            }


        }
    },
    async saveQueue(): Promise<void> {
        const songs: Array<Parameter> = [];

        console.log(this.queue);

        this.queue.forEach((song: Song, idx: number): void => {
            if (idx === 0) {
                songs.push({key: "current", value: song.id})
                songs.push({key: "id", value: song.id})
                // Add Progress within current song
            }
            else {
                songs.push({key: "id", value: song.id})
            }
        })

        const data = await OpenSubsonic.get("savePlayQueue", songs);
        if (data) {
            await this.getPlayQueue();
        }
    }
})

